#import "NeuronView.h"
#import "Neuron.h"
#import "Random.h"
#import "Network.h"
#import "SpikeArray.h"
#import "Spikes.h"

@implementation NeuronView

- (id) initWithFrame:(NSRect)frameRect
{
	int i ;
	if ((self = [super initWithFrame:frameRect]) != nil) {
		
		// Add initialization code here
		NonDefaultNeuronParamFile = nil ;
		NeuralNetwork = [[Network new] init: self] ; // initialise Neural Network
		Rng = [Random1 new] ;
		[self setNewRngSeed: nil] ;
		[NeuralNetwork initSetRng: Rng] ;
		startdisplaytime = 0 ;
		// initialise parameter keys
		SimParamnames = [NSMutableDictionary dictionary] ;
		NeuronParamnames = [NSMutableDictionary dictionary] ;
		NoiseParamnames = [NSMutableDictionary dictionary] ;

		[SimParamnames retain] ;
		[NeuronParamnames retain] ;
		[NoiseParamnames retain] ;
		[NeuralNetwork retain] ;
/*		printf("%s%s",[[Paramnames objectAtIndex: 2] cString] , "\n") ;
		printf("%s%s",[[Paramnames objectAtIndex: 0] cString] , "\n") ;
		// supply this MutableArray to NeuralNetwork */
		// line below may well be redundant
		[NeuralNetwork supplyParameters: SimParamnames: NeuronParamnames: NoiseParamnames] ;
		// Zeroise noise parameters
		for (i=0;i<3;i++)
			 [[NoiseParams cellAtIndex: i] setFloatValue: 0] ;
		[self readNoiseParams: NoiseParams] ;
		

	}
	return self;
}

-(void) fillBezierPath
{	
	NSRect therect ;
	NSString *s1, *s2 ;
	int i, n_neuron, ninput ;

	spikearray spikes, thisspike ;
	float spikeheight ;
	float tspike ;
	NSPoint linebottom, linetop ;
	
	simduration = enddisplaytime - startdisplaytime ;
	therect = [self bounds] ;
	stepwidth = therect.size.width/simduration ;
	// how many neurons are there?
	s1 = [SimParamnames valueForKey: [NSString stringWithCString: "Total Neurons"]] ;
	if (displayInputs)	// display both input and normal neurons	
		n_neuron = [s1 intValue] ;
	else
	{
		s2 = [SimParamnames valueForKey: [NSString stringWithCString: "Input Neurons"]] ;
		n_neuron = [s1 intValue] - [s2 intValue] ;
	}
	spikeheight = therect.size.height/(n_neuron + 2) ;	
	
	// declare baselines here, now we know the value of n_neuron
	float baselines[n_neuron] ;
	
	for (i=0;i<n_neuron;i++)
		baselines[i] = therect.size.height*((float)i/(float)(n_neuron+1)) ;
	// get the spikes
	spikes = [NeuralNetwork returnSpikes] ;
	thisspike = spikes ;
	// find out which are input neurons
	ninput = [NeuralNetwork getNumberOfInputNeurons] ;
	// create the path
	for (i = 0;i<nspikes;i++)
	{
		tspike = thisspike->timeofspike ;
		if ((tspike >= startdisplaytime) || (tspike <= enddisplaytime))
		{
			if (displayInputs || (thisspike->neuron >= ninput))
			{
				// move to bottom
				if (displayInputs)
					linebottom.y = baselines[thisspike->neuron] ;
				else
					linebottom.y = baselines[thisspike->neuron - ninput ] ;
				linebottom.x = (tspike - startdisplaytime) * stepwidth ; // fixed Aug24 2004 LSS
				linetop = linebottom ;
				linetop.y = linebottom.y + spikeheight ;
				// printf("%s%f%s%f%s", "line ", linebottom.x, " ", linebottom.y, "\n") ;
				// printf("%s%f%s%f%s", "line ", linetop.x, " ", linetop.y, "\n") ;
				// 
				[p1 moveToPoint: linebottom]  ;
				[p1 lineToPoint: linetop] ;
				
			}
			thisspike++ ;
		}
	}
	
}

- (void)drawRect:(NSRect)rect
{
	// new version 1 4 2008: bezier path is already drawn elsewhere.
	// all we do is set the colour and stroke it
	// results in much faster redrwing of the window.
	
	[[NSColor whiteColor] set];
    NSRectFill([self bounds]); 
	// draw X time scale
	
	// draw the spikes
	[[NSColor blackColor] set];
	
	[p1 stroke] ; // one huge bezier path
	image = rect; // for use in saving pdf
}


- (id) setDisplayTime: sender
// update the start and end display times from the DisplayTimes object
{
	startdisplaytime = [[DisplayTimes cellAtIndex: 0] floatValue] ;
	enddisplaytime = [[DisplayTimes cellAtIndex: 1] floatValue] ;
	return sender ;
}

// following 3 methods are for User selection of what to display
- (void)mouseDown:(NSEvent *)theEvent
{
	NSPoint mouseLoc;
	BOOL isInside = YES ;
	if ([theEvent clickCount] > 1)
		return ; // ignore doubleclicks
	else
	{
		mouseLoc = [self convertPoint: [theEvent locationInWindow] fromView: nil] ; 
		isInside = [self mouse:mouseLoc inRect: image] ; // is event inside the View?
		if (isInside)
		{
			[[DisplayTimes cellAtIndex: 0] setFloatValue: startdisplaytime + mouseLoc.x / stepwidth] ;
		}
	}
	return;
}

- (void)mouseDragged:(NSEvent *)theEvent
{
	return ;
}

- (void)mouseUp:(NSEvent *)theEvent
{	NSPoint mouseLoc;
	float endpoint ;
	BOOL isInside = YES ;
	if ([theEvent clickCount] > 1)
		return ; // ignore doubleclicks
	else
	{
		mouseLoc = [self convertPoint: [theEvent locationInWindow] fromView: nil] ; 
		isInside = [self mouse:mouseLoc inRect: image] ; // is event inside the View?
		if (isInside)
		{
			endpoint = startdisplaytime + mouseLoc.x / stepwidth ;
			if (endpoint > [[DisplayTimes cellAtIndex: 0] floatValue]) // must have end > start!
			[[DisplayTimes cellAtIndex: 1] setFloatValue: startdisplaytime + mouseLoc.x / stepwidth] ;
			[[DisplayTimes cellAtIndex: 2] setFloatValue: 
				(endpoint - [[DisplayTimes cellAtIndex: 0] floatValue])] ;
		}
	}
	
	return;
}

- (id) setupNeuralNetwork: sender 
{
	int displayAll ;
	// set the network up	
	// do we keep non-input neuron spikes?
	displayAll = [[DisplayRadio selectedCell] tag] ;
	displayInputs = displayAll ;
	
	// temp
	// NSLog(@"NeuronView.setupNeuralNetwork: displayAll = %i", displayAll) ;
	[self getSTDPParameters: sender] ;
	[self readSimParams: SimParams ] ;
	[self readNeuronParams: NeuronParams] ;
 
	[NeuralNetwork setup: MAXSPIKES : displayAll: SimParamnames : NeuronParamnames : NoiseParamnames  ] ; // set up the neurons
	if (SynapseFile != nil)
	{
		[NeuralNetwork setAdaptiveParams: adaptparams] ;
		[NeuralNetwork setSynapses: SynapseFile] ; // set up the synapses
	}
	if (inputsource == 1) // input from MatLab output file
		[NeuralNetwork setInputSpikes: InputSpikeFile] ; // set up the inputs
	if (inputsource == 2) // input is archived output from an earlier run
		[NeuralNetwork setInputSpikesFromArchive: ArchiveInputSpikes] ;
	if (inputsource == 3) // input is plain <neuron number> <time>
		[NeuralNetwork setSpikeInput: SpikeInputFile] ;
// use 0 and duration as the initial times for display
	startdisplaytime = 0 ;
	[[DisplayTimes cellAtIndex: 0] setFloatValue: startdisplaytime] ;
	enddisplaytime = [[SimParamnames valueForKey: [NSString stringWithCString: "Duration"]] floatValue ];
	[[DisplayTimes cellAtIndex: 1] setFloatValue: enddisplaytime] ;
	
	// 
	// should enable various save elements in menu here 
	return self ;
}


- (id) start:(id) sender
{   
	// start up the neural network simulation in NeuralNetwork
	// NeuralNetwork start returns a 2 by X array, with elements
	// neuron number, time
	//
	// allocate space each time start is called
	NSAutoreleasePool *SubPool_view ;
	SubPool_view = [[NSAutoreleasePool alloc] init] ;
	// create p1 bezier path
	if (p1 != nil)
	{	// added 1 4 2008
		// clear the bezier path
		[p1 removeAllPoints] ;
		[p1 release] ;
	}
	p1 = [NSBezierPath bezierPath] ;
	[p1 retain] ;
	nspikes = [NeuralNetwork start] ;
	[[SpikesGenerated cellAtIndex:0] setIntValue: nspikes] ;
	
	// now fill the bezier path
	[self fillBezierPath] ;
	
	[self setNeedsDisplay:YES]; // display the spikes
	[SubPool_view release] ;
	return self ;
}

- (id) reDisplay: sender
{	int displayAll ;
	// set up the network, butn not the files, nor the display times	
	// do we keep non-input neuron spikes?
	displayAll = [[DisplayRadio selectedCell] tag] ;
	displayInputs = displayAll ;
	[self setDisplayTime: sender] ;
	
	// added 1 4 2008 LSS
	// clear the bezier path
	if ([p1  elementCount] > 0)
		[p1 removeAllPoints] ;
	// now fill the bezier path
	[self fillBezierPath] ;
	
	[self setNeedsDisplay:YES]; // display the spikes
	return self ;
}

- (id) readNeuronParams: sender 
{ // reads the neuron parameters from the InfoParams panel into the ParamNames object
	// printf("%s", "in readParams \n") ;
	// 2nd try: can we semi-automate filling ParamNames?
	int i ;
	for (i=0;i<[sender numberOfRows];i++)
	{
		[NeuronParamnames setObject: [[sender cellAtIndex:i] stringValue] forKey: [[sender cellAtIndex:i] title]] ;
	}
	// debug
	// [self displayDictionary : NeuronParamnames] ;
	[NeuralNetwork applyNeuronParameters: NeuronParamnames] ; 

	return self ;
}

- (id) setNonDefaultNeuronParamFile: sender
{
	// selects a file containing the parameters for all the neurons that do not conform
	// to the default setting
	int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"neurons"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;
	
	// allow re-use of the last directory used
	if (LastInputSpikesDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastInputSpikesDirectory ;
	NSLog(@"The directory is %@", Directory) ;
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setTitle: @"Set non default neuron parameter file"] ;
    result = [oPanel runModalForDirectory: Directory
									 file:nil types:fileTypes];
    if (result == NSOKButton) {
		inputsource = 1 ; // mark as MatLab input
		LastInputSpikesDirectory = [oPanel directory] ;
		[LastInputSpikesDirectory retain] ;
		// printf("%s\n", [LastDirectory cString]) ;
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) {
            NonDefaultNeuronParamFile = [filesToOpen objectAtIndex:i];
        }
    }
	// debug info: what's in d1
	// NSLog(@"Input Spike file is %@", InputSpikeFile) ;
	[NonDefaultNeuronParamFile retain] ;
	return self ;
}

- (NSString *) getNonDefaultNeuronParamFile 
{
	// return the non default neuron parameter file name
	return NonDefaultNeuronParamFile ;
}

- (id) readNoiseParams: sender 
{ // reads the NoiseParams panel
	int s1, s2, s3 ;
	int i ;
	for (i=0;i<[sender numberOfRows];i++)
	{
		[NoiseParamnames setObject: [[sender cellAtIndex:i] stringValue] forKey: [[sender cellAtIndex:i] title]] ;
	}
	// debug
	// [self displayDictionary : NoiseParamnames] ;
	// and get the seeds back too
	[Rng getSeeds: &s1: &s2: &s3] ;
	[[NoiseSeeds cellAtIndex: 0] setIntValue : s1] ;
	[[NoiseSeeds cellAtIndex: 1] setIntValue : s2] ;
	[[NoiseSeeds cellAtIndex: 2] setIntValue : s3] ;
	return self ;
}

- (id) setNewRngSeed: sender {
	int s1, s2, s3 ;
// sets a new seed for the Random Number Generator
	[Rng newSeeds] ;
// and get the seeds back too
	[Rng getSeeds: &s1: &s2: &s3] ;
	[[NoiseSeeds cellAtIndex: 0] setIntValue : s1] ;
	[[NoiseSeeds cellAtIndex: 1] setIntValue : s2] ;
	[[NoiseSeeds cellAtIndex: 2] setIntValue : s3] ;	
	return self ;
}

- (id) readSimParams: sender 
{
	int i ;
	for (i=0;i<[sender numberOfRows];i++)
	{
		[SimParamnames setObject: [[sender cellAtIndex:i] stringValue] forKey: [[sender cellAtIndex:i] title]] ;
	}
	//debug
	// [self displayDictionary : SimParamnames] ;
	return self ;
}

- getSTDPParameters: sender
{
	// get the STDP parameters from the panel
	adaptparams.prepostNN = ! [[PrePostNN  selectedCell] tag] ;
	adaptparams.postpreNN = ! [[PostPreNN  selectedCell] tag] ;
	adaptparams.LTPwinner = ! [[LTPWins  selectedCell] tag] ;
	adaptparams.prepostadditive = ! [[PrePostAdd  selectedCell] tag] ;
	adaptparams.postpreadditive = ! [[PostPreAdd  selectedCell] tag] ;
	adaptparams.prepostTau = [[TauValues cellAtIndex: 0] floatValue] ;
	adaptparams.postpreTau =[[TauValues cellAtIndex: 1] floatValue]  ;
	adaptparams.prepostDeltaWeight = [[WeightValues cellAtIndex: 0] floatValue];
	adaptparams.postpreDeltaWeight = [[WeightValues cellAtIndex: 1] floatValue] ;
	adaptparams.maxallowedweight = [[WeightBoundaries cellAtIndex: 0] floatValue] ;
	adaptparams.minallowedweight = [[WeightBoundaries cellAtIndex: 1] floatValue] ;
	adaptparams.zxpermitted = ! [[ZxPermission  selectedCell] tag] ;
	adaptparams.zeroabsorbing = ! [[ZeroAbsorbing  selectedCell] tag] ;
	adaptparams.antihebbianforinhibitory = ! [[AntiHebbianInhibitory  selectedCell] tag] ;
	return self ;
}

- displayDictionary: (NSDictionary *) Dict1
// display a dictionary
{
	int nelements ;
	NSEnumerator *enumerator ;
	nelements = [Dict1 count] ;
	NSLog(@"NeuronView: displayDictionary no of elements = %i", nelements) ;
	enumerator = [Dict1 keyEnumerator];
	id key, valueforkey ;
	
	while ((key = [enumerator nextObject])) {
		/* code that uses the returned key */ 
		valueforkey = [Dict1 valueForKey: key] ;
		NSLog( @"displayDictionary key = %@", key) ;
		NSLog( @"value for key = %@", valueforkey) ;
	}
	return self;
}
	
- (id) showInfoPanel: sender
{
	[InfoPanel makeKeyAndOrderFront: self] ;
	return self ;
}

- (id) showSTDPPanel: sender
{
	[STDPParametersPanel makeKeyAndOrderFront: self] ;
	return self ;
}

- (id) showNeuronParameters: sender 
{
	int i ;
	id key ;
	[InfoPanel makeKeyAndOrderFront: self] ;
	
	// [self displayDictionary:SimParamnames] ;
	
	// go through NeuronParams and use the legend to select the value
	for (i=0;i<[NeuronParams numberOfRows];i++)
	{
		key = [[NeuronParams cellAtIndex:i] title] ;
		// printf("%s%s%s", "showSimParameters: ", [[[SimParams cellAtIndex:i] stringValue] cString], "\n") ; 
		// printf("%s%s%s", "showSimParameters: ", 
		// 	[[SimParamnames valueForKey:key] cString], "\n" ) ;
		[[NeuronParams cellAtIndex: i] setObjectValue:
			[NeuronParamnames valueForKey: key ] ]; 
	}
	return self ;
}

- (id) saveNeuronParams: sender 
{
	
	NSSavePanel *sp;
	int runResult;
	// store the Parameters object, SimParamnames using SavePanel to get the filename
	//
	/* create or get the shared instance of NSSavePanel */
	sp = [NSSavePanel savePanel];
	/* set up new attributes */
	// [sp setAccessoryView: newView];
	[sp setRequiredFileType:@"NeuronParams"];
	[sp setTitle: @"Save neuron parameters"] ;
	/* display the NSSavePanel */
	runResult = [sp runModalForDirectory:NSHomeDirectory() file:@"file"];
	/* if successful, save file under designated name */
	if (runResult == NSOKButton) 
	{
		if (![NeuronParamnames writeToFile:[sp filename] atomically:YES])
			NSBeep();
	}
	return self ;
}


- (id) loadNeuronParams: sender 
{
	// first load the dictionary up from a file
	// then place the values where they belong
    int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"NeuronParams"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;
	
	// allow re-use of the last directory used
	if (LastDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastDirectory ;
    [oPanel setAllowsMultipleSelection:YES];
	[oPanel setTitle: @"Load Default Neuron Parameters"] ;
    result = [oPanel runModalForDirectory: Directory
									 file:nil types:fileTypes];
    if (result == NSOKButton) {
		LastDirectory = [oPanel directory] ;
		[LastDirectory retain] ;
		// printf("%s\n", [LastDirectory cString]) ;
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) {
            NSString *aFile = [filesToOpen objectAtIndex:i];
            NeuronParamnames = [NeuronParamnames initWithContentsOfFile: aFile];
        }
    }
	// debug info: what's in d1
	NSLog(@"NeuronParamnames contains %i values", [NeuronParamnames count]) ;
	// store the parameters
	// [NeuralNetwork applyNeuronParameters: NeuronParamnames] ; 
	// update display
	[self showNeuronParameters: nil] ; 
	return self ;
}

- (id) showNoiseParameters: sender 
{
	int i ;
	id key ;
	[InfoPanel makeKeyAndOrderFront: self] ;
	
	// [self displayDictionary:SimParamnames] ;
	
	// go through NeuronParams and use the legend to select the value
	for (i=0;i<[NoiseParams numberOfRows];i++)
	{
		key = [[NoiseParams cellAtIndex:i] title] ;
		// printf("%s%s%s", "showSimParameters: ", [[[SimParams cellAtIndex:i] stringValue] cString], "\n") ; 
		// printf("%s%s%s", "showSimParameters: ", 
		// 	[[SimParamnames valueForKey:key] cString], "\n" ) ;
		[[NoiseParams cellAtIndex: i] setObjectValue:
			[NoiseParamnames valueForKey: key ] ]; 
	}
	return self ;
}


- (id) saveNoiseParams: sender 
{
	
	NSSavePanel *sp;
	int runResult;
	// store the Parameters object, NoiseParamnames using SavePanel to get the filename
	//
	/* create or get the shared instance of NSSavePanel */
	sp = [NSSavePanel savePanel];
	/* set up new attributes */
	// [sp setAccessoryView: newView];
	[sp setRequiredFileType:@"NoiseParams"];
	[sp setTitle: @"Save noise parameters"] ;
	/* display the NSSavePanel */
	runResult = [sp runModalForDirectory:NSHomeDirectory() file:@"file"];
	/* if successful, save file under designated name */
	if (runResult == NSOKButton) 
	{
		if (![NoiseParamnames writeToFile:[sp filename] atomically:YES])
			NSBeep();
	}
	return self ;
}


- (id) loadNoiseParams: sender 
{
	// first load the dictionary up from a file
	// then place the values where they belong
    int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"NoiseParams"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;
	
	// allow re-use of the last directory used
	if (LastDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastDirectory ;
    [oPanel setAllowsMultipleSelection:YES];
	[oPanel setTitle: @"Load Noise Parameters"] ;
    result = [oPanel runModalForDirectory: Directory
									 file:nil types:fileTypes];
    if (result == NSOKButton) {
		LastDirectory = [oPanel directory] ;
		[LastDirectory retain] ;
		// printf("%s\n", [LastDirectory cString]) ;
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) {
            NSString *aFile = [filesToOpen objectAtIndex:i];
            NoiseParamnames = [NoiseParamnames initWithContentsOfFile: aFile];
        }
    }
	// debug info: what's in d1
	NSLog(@"NoiseParamnames contains %i values", [NoiseParamnames count]) ;
// store the parameters
[NeuralNetwork applyNoiseParameters: NoiseParamnames] ; 
// update display
[self showNoiseParameters: nil] ; 
return self ;
}

- (id) showSynapseInfoPanel: sender 
{
	[SynapseInfoPanel makeKeyAndOrderFront: self] ;
	[NeuralNetwork displaySynapses:SynapseTextView] ; // all the work is done in NeuralNetwork
	return self ;
}

- (id) clearSynapseInfo: sender
{
	NSRange endRange;

	endRange.length = [[SynapseTextView textStorage] length];
	endRange.location = 0;
	[SynapseTextView replaceCharactersInRange:endRange withString:[NSString string]];
	return self ;
}

- (id) showSimParameters: sender 
{
	int i ;
	id key ;
	[InfoPanel makeKeyAndOrderFront: self] ;
	
	// [self displayDictionary:SimParamnames] ;
	
	// go through NeuronParams and use the legend to select the value
	for (i=0;i<[SimParams numberOfRows];i++)
	{
		key = [[SimParams cellAtIndex:i] title] ;
		// printf("%s%s%s", "showSimParameters: ", [[[SimParams cellAtIndex:i] stringValue] cString], "\n") ; 
		// printf("%s%s%s", "showSimParameters: ", 
		// 	[[SimParamnames valueForKey:key] cString], "\n" ) ;
		[[SimParams cellAtIndex: i] setObjectValue:
			[SimParamnames valueForKey: key ] ]; 
	}
	return self ;
}

- (id) saveSimParams: sender 
{

	NSSavePanel *sp;
	int runResult;
	// store the Parameters object, SimParamnames using SavePanel to get the filename
	//
	/* create or get the shared instance of NSSavePanel */
	sp = [NSSavePanel savePanel];
	/* set up new attributes */
	// [sp setAccessoryView: newView];
	[sp setRequiredFileType:@"SimParams"];
	[sp setTitle: @"Save simulation parameters"] ;
	/* display the NSSavePanel */
	runResult = [sp runModalForDirectory:NSHomeDirectory() file:@"file"];
	/* if successful, save file under designated name */
	if (runResult == NSOKButton) 
	{
		if (![SimParamnames writeToFile:[sp filename] atomically:YES])
          NSBeep();
	}
	return self ;
}

	
- (id) loadSimParams: sender 
{
	// first load the dictionary up from a file
	// then place the values where they belong
    int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"SimParams"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;
	// temp
	// NSMutableDictionary *xxx ;

	// allow re-use of the last directory used
	if (LastDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastDirectory ;
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setTitle: @"Load Simulation Parameters"] ;
    result = [oPanel runModalForDirectory: Directory
                    file:nil types:fileTypes];
    if (result == NSOKButton) {
		LastDirectory = [oPanel directory] ;
		[LastDirectory retain] ;
		printf("%s\n", [LastDirectory UTF8String]) ;
        NSArray *filesToOpen = [oPanel filenames];
        int i, count1 = [filesToOpen count];
        for (i=0; i<count1; i++) {
            NSString *aFile = [filesToOpen objectAtIndex:i];
			// printf("%s\n", [aFile UTF8String]) ;
			[SimParamnames removeAllObjects] ;
            SimParamnames = [SimParamnames initWithContentsOfFile: aFile];
			// NSLog(@"SimParamnames contains %d values", [SimParamnames count]) ;
        }
    }
	// debug info: what's in d1
	NSLog(@"SimParamnames contains %d values", [SimParamnames count]) ;
	// store the parameters
	[NeuralNetwork applySimParameters: SimParamnames] ; 
	// update display
	[self showSimParameters: nil] ; 
	return self ;
}

- setSynapseFile: sender
{ 
	// returns the string for the synapse file: opened in Network
	int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"synapse"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;
	
	// allow re-use of the last directory used
	if (LastDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastDirectory ;
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setTitle: @"Set Synapse File Name"] ;
    result = [oPanel runModalForDirectory: Directory
									 file:nil types:fileTypes];
    if (result == NSOKButton) {
		LastDirectory = [oPanel directory] ;
		[LastDirectory retain] ;
		// printf("%s\n", [LastDirectory cString]) ;
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) {
            SynapseFile = [filesToOpen objectAtIndex:i];
        }
    }
	// debug info: what's in d1
//	printf("%s%s%s","Synapse file is ", [SynapseFile cString], "  \n") ;
	[SynapseFile retain] ;
	[[FileNameDisplay  cellAtIndex: 0] setObjectValue: SynapseFile] ;	

	return self ;
}
	
- (id) saveSynapses: sender 
{
	// save the (updated) synapses into a file
	NSSavePanel *sp;
	int runResult;
	NSString  *Directory ;
	//
	// allow re-use of the last directory used
	if (LastDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastDirectory ;
	
	/* create or get the shared instance of NSSavePanel */
	sp = [NSSavePanel savePanel];
	/* set up new attributes */
	// [sp setAccessoryView: newView];
	[sp setRequiredFileType:@"synapse"];
	[sp setTitle: @"Save synapses"] ;
	/* display the NSSavePanel */
	runResult = [sp runModalForDirectory:Directory file:@"file"];
	/* if successful, save file under designated name */
	if (runResult == NSOKButton) 
	{
		// and save the synapses
		[NeuralNetwork saveSynapses: [sp filename]] ;
	}
	
	return self ;
}


- (NSString *) getSynapseFile 
{
	return SynapseFile ;
}

- setInputSpikesFile: sender
{ 
	// used for Matlab Generated files
	// returns the string for the synapse file: opened in Network
	int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"inspikes"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;
	
	// allow re-use of the last directory used
	if (LastInputSpikesDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastInputSpikesDirectory ;
	NSLog(@"The directory is %@", Directory) ;
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setTitle: @"Set audio input spike file"] ;
    result = [oPanel runModalForDirectory: Directory
									 file:nil types:fileTypes];
    if (result == NSOKButton) {
		inputsource = 1 ; // mark as MatLab input
		LastInputSpikesDirectory = [oPanel directory] ;
		[LastInputSpikesDirectory retain] ;
		// printf("%s\n", [LastDirectory cString]) ;
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) {
            InputSpikeFile = [filesToOpen objectAtIndex:i];
        }
    }
	// debug info: what's in d1
	[[FileNameDisplay  cellAtIndex: 1] setObjectValue: InputSpikeFile] ;
	// NSLog(@"Input Spike file is %@", InputSpikeFile) ;
	[InputSpikeFile retain] ;
	return self ;
}

- setSpikeInputFile: sender
{ 
	// used for hand generated spike input files of the form
	// <neuronnumber> <time>
	// returns the string for the synapse file: opened in Network
	int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"spikesin"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;
	
	// allow re-use of the last directory used
	if (LastInputSpikesDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastInputSpikesDirectory ;
	NSLog(@"The directory is %@", Directory) ;
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setTitle: @"Set simple spike file"] ;
    result = [oPanel runModalForDirectory: Directory
									 file:nil types:fileTypes];
    if (result == NSOKButton) {
		inputsource = 3 ; // mark as MatLab input
		LastInputSpikesDirectory = [oPanel directory] ;
		[LastInputSpikesDirectory retain] ;
		// printf("%s\n", [LastDirectory cString]) ;
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) {
            SpikeInputFile = [filesToOpen objectAtIndex:i];
        }
    }
	// debug info: what's in d1
	// display file name
	[[FileNameDisplay  cellAtIndex: 1] setObjectValue: SpikeInputFile] ;
	// NSLog(@"Spike Input  file (plain input) is %@", SpikeInputFile) ;
	[SpikeInputFile retain] ;
	return self ;
}


- (NSString *) getInputSpikesFile 
{
	// MATLAB based input
	return InputSpikeFile ;
}

- (NSString *) getSpikeInputFile 
{
	// plain input
	return SpikeInputFile ;
}

- (id) saveSpikesGeneratedHumanReadable: sender 
// save the generated spikes in a file in a simple (but verbose) format
// added 11 11 2004
{
	NSSavePanel *sp;
	int runResult;
	NSString *Directory ;
	spikearray spikes ;
	SpikeArray * SpikeArrayObject ;
	Spikes *SpikesObject ;
	
	// use NSSavePanel to get filename
	/* create or get the shared instance of NSSavePanel */
	sp = [NSSavePanel savePanel];
	// allow re-use of the last directory used
	if (LastSpikeOutputDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastSpikeOutputDirectory ;
	
	/* set up new attributes */
	// [sp setAccessoryView: newView];
	[sp setRequiredFileType:@"spikesin"];
	[sp setTitle:@"Save spikes generated (human readable)"] ;
	/* display the NSSavePanel */
	runResult = [sp runModalForDirectory:Directory file:@"file"];
	/* if successful, save file under designated name */
	if (runResult == NSOKButton) 
	{
		LastSpikeOutputDirectory = [sp directory] ;
		[LastSpikeOutputDirectory retain] ;
		
		// Build a new object which had pointers to all the data required, 
		// and then call method inside this to write out the spikes using C file handling
		// get the spikes
		spikes = [NeuralNetwork returnSpikes] ;
		// set up the SpikesArray object
		SpikeArrayObject = [[SpikeArray new] initWithSpikearrayAndNumSpikes: spikes
															 numberofspikes: nspikes] ;
		// set up the Spikes object
		SpikesObject = [[Spikes new] initWithSpikeArrayAndParams: SpikeArrayObject params: SimParamnames] ;
		// and archive it to file
		// now call the method to write it out
		[SpikesObject saveSpikesHumanReadable: [sp filename]] ;
		// 
	}
	return self ;
}

- (id) saveSpikesGenerated: sender 
// save the generated spikes in a file: uses Obj C archiving
{
	NSSavePanel *sp;
	int runResult;
	NSString *Directory ;
	spikearray spikes ;
	SpikeArray * SpikeArrayObject ;
	Spikes *SpikesObject ;
	
	// use NSSavePanel to get filename
	/* create or get the shared instance of NSSavePanel */
	sp = [NSSavePanel savePanel];
	// allow re-use of the last directory used
	if (LastSpikeOutputDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastSpikeOutputDirectory ;

	/* set up new attributes */
	// [sp setAccessoryView: newView];
	[sp setRequiredFileType:@"outspikes"];
	[sp setTitle:@"Archive spikes generated"] ;
	/* display the NSSavePanel */
	runResult = [sp runModalForDirectory:Directory file:@"file"];
	/* if successful, save file under designated name */
	if (runResult == NSOKButton) 
	{
		LastSpikeOutputDirectory = [sp directory] ;
		[LastSpikeOutputDirectory retain] ;

		// Would an alternative be to build a new object which had pointers to all the data required, 
		// and then archive this object?: let's try it
		// get the spikes
		spikes = [NeuralNetwork returnSpikes] ;
		// set up the SpikesArray object
		SpikeArrayObject = [[SpikeArray new] initWithSpikearrayAndNumSpikes: spikes
															 numberofspikes: nspikes] ;
		// set up the Spikes object
		SpikesObject = [[Spikes new] initWithSpikeArrayAndParams: SpikeArrayObject params: SimParamnames] ;
		// and archive it to file
		if ([NSKeyedArchiver archiveRootObject: SpikesObject toFile: [sp filename]])
			NSLog(@"NeuronView.saveSpikesGenerated SpikesObject archived successfully") ;
		else
			NSLog(@"NeuronView.saveSpikesGenerated SpikesObject failed to archive") ;
		// 
	}
	return self ;
}

- (id) loadSpikesGeneratedEarlier: sender 
{
	// first select the file to be loaded
	// first load the dictionary up from a file
	// then place the values where they belong
    int result;
    NSArray *fileTypes = [NSArray arrayWithObject:@"outspikes"];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	NSString  *Directory ;

	
	// allow re-use of the last directory used
	if (LastDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastDirectory ;
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setTitle: @"Load archived spikes"] ;
    result = [oPanel runModalForDirectory: Directory
									 file:nil types:fileTypes];
    if (result == NSOKButton) {
		inputsource = 2 ; // mark as input from archived file
		LastDirectory = [oPanel directory] ;
		[LastDirectory retain] ;
		// now keep the filename ;
		ArchiveInputSpikes = [oPanel filename];
		[ArchiveInputSpikes retain] ;
	}
	[[FileNameDisplay  cellAtIndex: 1] setObjectValue: ArchiveInputSpikes] ;	
	return self ;
}

- (id) savePDFDisplay: sender
{
// save the .pdf inside the display to a file
	// first find the file name
	NSSavePanel *sp;
	int runResult;
	NSString  *Directory ;
	//
	// allow re-use of the last directory used
	if (LastDirectory == nil) 
		Directory = NSHomeDirectory() ;
	else Directory = LastDirectory ;
	
	/* create or get the shared instance of NSSavePanel */
	sp = [NSSavePanel savePanel];
	/* set up new attributes */
	// [sp setAccessoryView: newView];
	[sp setRequiredFileType:@"pdf"];
	[sp setTitle: @"Save View as .pdf"] ;
	/* display the NSSavePanel */
	runResult = [sp runModalForDirectory:Directory file:@"file"];
	/* if successful, save file under designated name */
	if (runResult == NSOKButton) 
	{
		// and save the pdf inside the view
		if
		([[self dataWithPDFInsideRect: [self bounds]] writeToFile: [sp filename] atomically:YES])
			NSLog(@"NeuronView: savePDFDisplay: pdf written to file %@", [sp filename]) ;
	}
	return self ;
}
	
// delegate methods 

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	// NSLog(@"NeuronView.applicationDidFinishLaunching\n") ;
	// put sensible default values into the noise cells
	int i ;
	float taudefault;
	taudefault = 520.0 ;
	for (i=0;i<3;i++)
		[[NoiseParams cellAtIndex: i] setFloatValue: 0] ;
	[self readNoiseParams: NoiseParams] ;
	// and into the weight change panel
	[[WeightBoundaries cellAtIndex: 0] setFloatValue: 1.0] ;
	[[WeightBoundaries cellAtIndex: 1] setFloatValue: -1.0] ;	
	[[TauValues cellAtIndex: 0] setFloatValue: taudefault] ;
	[[TauValues cellAtIndex: 1] setFloatValue: taudefault] ;
	[[WeightValues cellAtIndex: 0] setFloatValue: 0] ;
	[[WeightValues cellAtIndex: 1] setFloatValue: 0] ;
	
	// and display the parameter panels
	[self showInfoPanel: nil] ;
	[self showSTDPPanel: nil] ;
			 
}

@end
