//
//  Network.m
//  Neuron1
//
//  Created by Leslie Smith on Mon Jun 07 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "Network.h"
#import "Neuron.h"


@implementation Network

- initSetRng: rngid
/* set up random number generator class object */
{
	RandomNumberGenerator = rngid ;
	return self ;
}

- supplyParameters:  SP: NeP: NoP
/* get the parameter Mutable Array */
{
	SimParameters = SP ;
	NeuronParameters = NeP ;
	NoiseParameters = NoP ;
/*	[SimParameters retain] ;
	[NeuronParameters retain] ;
	[NoiseParameters retain] ;
	*/
	return self ;
}


- applySimParameters
{
	NSString *s1 ;
	int n_neuron_new ;
	// apply the simulation parameters
	// how many neurons are there?
	s1 = [SimParameters valueForKey: [NSString stringWithCString: "No of Neurons"]] ;
	n_neuron_new = [s1 intValue] ;
	if (n_neuron != n_neuron_new)
		printf("%s", "NeuralNetwork:applySimParameters. Can't change number of neurons") ;
	
	// how long should simulation last ?
	s1 = [SimParameters valueForKey: [NSString stringWithCString: "Duration"]] ;
	duration = [s1 floatValue] ;
	// what is the timestep
	s1 = [SimParameters valueForKey: [NSString stringWithCString: "Timestep"]] ;
	timestep = [s1 floatValue] ;
	// how many simulation steps
	simsteps = round(duration/timestep) ;
	return self ;
}

- applyNeuronParameters
{
	// apply the simulation parameters
	NSString *s1 ;
	int i ;
	float zerolevel, initphase, tonicvalue,threshold, refperiod, dissipation, squarediss ;
	
	s1 = [NeuronParameters valueForKey: [NSString stringWithCString: "Zero Level"]] ;
	zerolevel = [s1 floatValue] ;
	s1 = [NeuronParameters valueForKey: [NSString stringWithCString: "Initial Ph"]] ;
	initphase = [s1 floatValue] ;
	s1 = [NeuronParameters valueForKey: [NSString stringWithCString: "Tonic"]] ;
	tonicvalue = [s1 floatValue] ;
	s1 = [NeuronParameters valueForKey: [NSString stringWithCString: "Threshold"]] ;
	threshold = [s1 floatValue] ;
	s1 = [NeuronParameters valueForKey: [NSString stringWithCString: "Ref. Period"]] ;
	refperiod = [s1 floatValue] ;
	s1 = [NeuronParameters valueForKey: [NSString stringWithCString: "Dissipation"]] ;
	dissipation = [s1 floatValue] ;
	s1 = [NeuronParameters valueForKey: [NSString stringWithCString: "Sq. Diss'n"]] ;
	squarediss = [s1 floatValue] ;
	// apply these values to the neurons in the network
	for (i=0;i<[NeuronArray count]; i++)
	{
		 [[NeuronArray objectAtIndex: i]  setRefPeriod: refperiod] ;	// before setting time step
		 [[NeuronArray objectAtIndex: i]  setTimeStep: timestep] ; // actually from SimParameters
		 [[NeuronArray objectAtIndex: i] setPhase: initphase] ;
		 [[NeuronArray objectAtIndex: i] setTonic: tonicvalue] ;
		 [[NeuronArray objectAtIndex: i] setThreshold: threshold] ;
		 [[NeuronArray objectAtIndex: i] setRefPeriod: refperiod] ;
		 [[NeuronArray objectAtIndex: i] setDissipation: dissipation] ;
		 [[NeuronArray objectAtIndex: i] setSquareDissipation: squarediss] ;
		 // temp
		 //  printf("%s%f%s","dissipation = ", dissipation, "\n") ;
	}
	// 
	return self ;
}

- applyNoiseParameters
{
	// apply the simulation parameters
	NSString *s1 ;
	int i ;
	float tonicnoise, threshnoise, activnoise ;
	
	s1 = [NoiseParameters valueForKey: [NSString stringWithCString: "Tonic N"]] ;
	tonicnoise = [s1 floatValue] ;
	s1 = [NoiseParameters valueForKey: [NSString stringWithCString: "Thresh N"]] ;
	threshnoise = [s1 floatValue] ;
	s1 = [NoiseParameters valueForKey: [NSString stringWithCString: "Activ N"]] ;
	activnoise = [s1 floatValue] ;
	// apply these values to the neurons in the network
	for (i=0;i<[NeuronArray count]; i++)
	{
		[[NeuronArray objectAtIndex: i]  tonicNoise: tonicnoise] ;	
		[[NeuronArray objectAtIndex: i]  thresholdNoise: threshnoise] ;	
		[[NeuronArray objectAtIndex: i]  activationNoise: activnoise] ;	
	}
	return self ;
}

- (id) setup: (int) maxspikes
{
	NSString *s1 ;
	int i ;
	
	if (SimParameters == nil)
		printf("%s", "Neuralnetwork: setup. SimParameters is nil. \n") ;
	
	// set up spike array
	if (spikesout != nil) free(spikesout) ;
	spikesout = (spikearray) malloc(sizeof(spiketype) * maxspikes) ;
	
	// how many neurons are there?
	s1 = [SimParameters valueForKey: [NSString stringWithCString: "No of Neurons"]] ;
	n_neuron = [s1 intValue] ;
	// how long should simulation last ?
	s1 = [SimParameters valueForKey: [NSString stringWithCString: "Duration"]] ;
	duration = [s1 floatValue] ;
	// what is the timestep
	s1 = [SimParameters valueForKey: [NSString stringWithCString: "Timestep"]] ;
	timestep = [s1 floatValue] ;
	// how many simulation steps
	simsteps = round(duration/timestep) ;
	
	// set up the network
	if (NeuronArray != nil) [NeuronArray release] ;
	NeuronArray = [NSMutableArray arrayWithCapacity: n_neuron] ;
	[NeuronArray retain] ;
	for (i=0;i<n_neuron;i++)
		[NeuronArray addObject: [[[Neuron new] init] initSetRng: RandomNumberGenerator]] ;

	[self applyNeuronParameters] ;
	[self applyNoiseParameters] ;
	
	

	
	if (s1 == nil)
		printf("%s","Network:setup  s1 is nil\n") ;
	else
		printf("%s%i%s", "Network: simsteps = ",simsteps, "\n") ;
	
	[self displayDictionary: SimParameters] ;
	return self ;
}

- (int) start
{
	int nspikes ; nspikes = 0 ;
	int i ;
	// run the simulation for simsteps time steps.
	NSEnumerator *enumerator = [NeuronArray objectEnumerator];
	Neuron *ThisNeuron;
	spikearray nextspike ;
	nextspike = spikesout ;
	int neuron_number ;
	
	nextspike = spikesout ; // point to start of spike array
	for ( i=0;i<simsteps;i=i+1)
	{
		neuron_number = 0 ;
		while (ThisNeuron = [enumerator nextObject]) {

			if ([ThisNeuron runStep: 0] == 1) // record spike times
			{
				nspikes++ ;
				nextspike->neuron = neuron_number;
				nextspike->timeofspike = i * timestep;
				nextspike++ ;
				/* if (nspikes == MAXSPIKES)
					printf("%s%i%s","Too many spikes", nspikes, 
					   ". Increase MAXSPIKES in NeuronView.h. \n") ; */

			}				
			// printf("%s%i%s", " Network:start timestep=", i, "\n") ;
			// printf("%s%i%s", " Network:start Neuron=", neuron_number, "\n") ;
			neuron_number++ ;
		}
		enumerator = [NeuronArray objectEnumerator]; // back to start of array
		// printf("%s%i%s","runstep = ", result, "\n") ;
	}

	printf("%s%i%s","Network:start. spike count = ", nspikes, "\n") ;
	return nspikes ;
}

- (spikearray) returnSpikes 
{
	return spikesout ;
}

- setSynapses: synapsefile
// read the synapse file (currently an NSString)
{
	FILE *sfile; // synapse file: text file
	syndatatype syn1 ;
	syndataptr syn1p , currsynptr ;
	
	// read the synapses into the synapse_interconnection structure
	sfile = fopen([synapsefile cString],"r") ;
	if (synapses != nil) free(synapses) ;
	while (fscanf(sfile, "%i%i%i%f",  &syn1.presyn, &syn1.postsyn, &syn1.syntype, &syn1.weight) != EOF)
	{
		// store values as a list
		syn1p = (syndataptr) malloc(sizeof(syndatatype)) ;
		*syn1p = syn1 ;
		syn1p -> nextsyn = nil ;
		if (synapses == nil)
		{
			synapses = syn1p ;
			currsynptr = synapses ;
		}
		else{
			currsynptr->nextsyn = syn1p ;
			currsynptr = syn1p ;
		}
	}
	// set up the synapses inside the neurons
	return self ;
}


- displayDictionary: Dict1
	// display a dictionary
{
	int nelements ;
	NSEnumerator *enumerator ;
	nelements = [Dict1 count] ;
	printf("%s%i%s", "Network: displayDictionary no of elements = ", nelements, "\n") ;
	enumerator = [Dict1 keyEnumerator];
	id key, valueforkey ;
	
	while ((key = [enumerator nextObject])) {
		/* code that uses the returned key */ 
		valueforkey = [Dict1 valueForKey: key] ;
		printf("%s%s", "displayDictionary key = ", [key  cString] ) ;
		printf("%s%s%s", " value for key = ", 
			   [valueforkey  cString] , "\n") ;
	}
	return self;
}


@end
