
#import <appkit/appkit.h>

#define DEFTHRESH 1.0
#define DEFDISS 0.0
#define DEFTIME 0.01
#define DEFMINACT 0.0
#define DEFREFPD 0.0 

@interface Neuron: NSObject
{
id Rng ; /* random number generator */
float activation; /* V-like state variable */
float threshold ; /* theta in equations */
float dissipation ; /* gamma in equations */
float timestep ; /* default timestep in seconds */
float initphase ; /* initial phase */
int output ; /* normally 0, 1 for one time step */
float minact ; /* minimum permitted activation */
float refractperiod ; /* refractory period */
int intrefperiod, refractcount ; /* for timing refractory period */
float tonicinput ; /* tonic input */
int nonoise ;
float tonicnoise, threshnoise, activationnoise ;
}

- initial: (float) newthresh: (float) newdiss: 
	(float) minactivation: (float) refpd ; /* initialise */
- initSetRng: rngid ; /* set up random number generator */
- setTimeStep: (float) stepsize; /* set timestep */
- setTonic: (float) tonicinput ;
- setPhase: (float) phasepoint ; /* set initial phase */
- setThreshold: (float) thresh ;
- setRefPeriod: (float) refpd ; 
- setDissipation: (float) diss ; 

- resetNeuron ; /* set activation and output back to 0 */

- noNoise ; /* set up absence of noise */
- tonicNoise: (float) level ; /* set up fractional noise on tonic input */
- thresholdNoise: (float) level ; /* set up noise on threshold */
- activationNoise: (float) level ; /* set up noise on activation */

- (float) getActivation; /* returns activation level */
- (int) getOutput ; /* returns output */
- (float) getTimeStep ; /* returns timestep in use last time step */
- (float) getThreshold ; /* return threshold */
- (float) getPhase ; /* return phase */
- (float) getTonic; /* returns tonic input level */
- (float) getPeriod ; /* returns calculated period */
- (float) getRefPeriod ; /* get refractory period */
- (float) getDissipation ; /* get dissipation */

/* run for 1 timestep */
/* returns output value */
- (int) runStep:  (float) otherinput ;

@end
