//
//  synapse.h
//  Neuron1
//
//  Created by Leslie Smith on Tue Jun 15 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "Neuron.h"
// import the synapse data header file
#import "syndata.h" 
// MAXTIMESTEPEFFECT is the maximal number of timesteps that the alpha function
// can last over
#define MAXTIMESTEPEFFECT 10000
// MAXCONCURRENTPRESYN is maximal number of concurrent presynaptic spikes
// needs to be > no of spikes in alphaeffect + delay
#define MAXCONCURRENTPRESYN 200
// MAXPRESYNDT is factor times prepostTau that is used to determine if
// t_fire - lastspiketime is too long ago for LTP
#define MAXPRESYNDT 5
// MAXPOSTSYNDT is factor times postpreTau that is used to determine if
// t_fire - lastspiketime is too long ago for LTD
#define MAXPOSTSYNDT 5
// MAXKEPTSPIKES is maximal number of kept spikes for use in classic pre->post STDP
#define MAXKEPTSPIKES 500

@interface synapse : NSObject {
	float weight ;
	int synapsetype ;
	Neuron *PresynNeuron ;
	int prenumber, postnumber ;
	Neuron *PostsynNeuron ;
	BOOL neverspiked ;
	double lastspiketimes[MAXCONCURRENTPRESYN] ; // when last spikes were received
	int lastspikesteps[MAXCONCURRENTPRESYN] ; // ditto in timesteps
	int presynspikes ; // number of current presynaptic spikes in progress
	float lastpostsyntime ; // time of last postsynaptic firing
	int lastpostsynstep ; // step number of last postsynaptic firing
	double delay ; // pre alpha function delay
	int delaysteps ; // ditto in steps
	BOOL resetContributionOnInputSpike ;
	float alpha ;
	float alphaarray[MAXTIMESTEPEFFECT] ;
	int lengthalphaeffect ;
	bool alpha_reset ; // YES if alpha function is to be reset on postsynaptic neuron firing
	bool do_reset_alpha ; // YES if alpha is to be reset this time
	bool isadaptive ;
	STDPinfo adaptvalues ; // holds adaptation values if synapse adaptive
	// last presynaptic info for NN system
	float lastusedspiketime ; // last spike to pass through
	int lastusedspikestep ; // ditto in sim steps
	// last presynaptic info for classic system (added 24 10 2004)
	float lastusedspiketimearray[MAXKEPTSPIKES] ; // last spike times
	int lastusedspikesteparray[MAXKEPTSPIKES] ; // last spike steps
	int lastusednumber ;
	BOOL postsynusedadaptively ; // set for use with LTPwinner in STDP
	BOOL postsynusedpresynaptically ; // for checking whether presynaptic pulses have been used for STDP
										// in a post -> pre situation
	BOOL postsynusedpostsynaptically ; // for checking whether presynaptic pulses have been used for STDP
										// in a pre->post situation

}
// setup etc.
- setWeight: (float) w ;
- setAlpha: (float) alph alphareset: (bool) alphareset; // set alpha and alpha resetting for alpha function
- computeSynapseEffects: (double) timestep;
- setSynapseType: (int) s ;
- setPresynNeuron: (Neuron *) NNN number: (int) preno ;
- setPostsynNeuron: (Neuron *) NNN number: (int) postno;
- setDelay: (double)d : (int) dsteps ;
- setAdaptive: (BOOL) adaptive:  (STDPinfo) adaptparams ;
- setResetContributionOnInputSpike: (BOOL) resetval ; // YES implies only 1 spike contributes
- (float) getWeight ;
- (int) getSynapseType ; // 1=non-dynamic 2=dynamic
- (syndatatype) getSynapseData ; // returns whole structure describing synapse configuration
- getPresynNeuron ;
- getPostsynNeuron ;

// actual usage
- presynFires: (float) t_fire : (int) stepfire; // spike arrives at t_fire
- (float) getSynapticContribution: (double) current_time: (int) stepno ; 
- postsynFires: (float) t_fire: (int) stepfire ; // for resetting alpha functions and STDP adjustment
- applySTDPPresynaptically : (float) presyntime ;
@end
